/*
        Copyright 2006 Bianca van Schaik

    This file is part of the KDECalendar widget.

    KDECalendar is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KDECalendar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KDECalendar; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
* Calendaring functions
*/

/* Global references to DOM nodes: */
var calendarTbody;
var dateInput;
var weekInput;

var choosemonth_input;
var choosemonth_span;

var chooseyear_input;
var chooseyear_span;
/* End DOM nodes */

/* Calendaring variables */
var currentDay;
var currentMonth;
var currentYear;

var todayDay;
var todayMonth;
var todayYear;

var monthNames = ["January", "February", "March", "April", "May", "June",
	"July", "August", "September", "October", "November", "December" ];

var monthDays = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];

var dayNames = [ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
	"Friday", "Saturday", "Sunday" ];

/**
 * Returns whether this is a leap year
 */
function isLeapYear(year) {
	if (year % 400 == 0) {
		return false;
	} else if (year % 4 == 0) {
		return true;
	}
	return false;
}

/**
 * Returns the number of days in the given month
 */
function getDaysInMonth(month, year) {
	month = (month + 12) % 12;
	if (month == 1 && isLeapYear(year)) {
		return 29;
	}
	return monthDays[month];
}

/**
 * Gets the day number of this day. Example, jan 1 is 1, feb 1 is 32
 */
function getDayNumber(year, month, day) {
	var totalDays = 0;
	for (i = 0; i < month; i++) {
		totalDays += monthDays[i];
	}
	if (month > 2 && isLeapYear(year)) {
		totalDays++;
	}
	totalDays += day;
	return totalDays;
}

/**
 * Gets the week number of this day
 */
function getWeekNumber(year, month, day) {
	var date = new Date(year, 0, 1);
	var totalDays = getDayNumber(year, month, day);
	
	totalDays += (date.getDay() + 2) % 7 - 3;
	return Math.ceil(totalDays / 7);
}

/**
 * Draws the current calendar, based on the values of currentYear, currentMonth
 */
/*
function drawCalendarX() {
	var startTimer = new Date();
	var html = '';
	// Figure out in which week the first of this month starts
	var weekNumber = getWeekNumber(currentYear, currentMonth, 1);
	// Figure out which number from the /previous/ month to start with
	var date = new Date(currentYear, currentMonth, 1);
	var weekday = ((date.getDay() + 6) % 7) + 1;
	var prevMonthDays = getDaysInMonth(currentMonth-1, currentYear); // don't worry about year, dec == 31 no matter what
	var monthDays = getDaysInMonth(currentMonth, currentYear);
	
	var startDate = prevMonthDays - weekday + 2;
	if (weekday == 1) { // monday, subtract 7
		startDate -= 7;
		weekNumber--;
	}
	var day = startDate;
	var inOldMonth = true;
	var inNewMonth = false;
	for (i = 0; i < 42; i++) {
		if (i % 7 == 0) {
			html += '<tr><th class="week">' + weekNumber + '</th>';
			weekNumber++;
		}
		if (inOldMonth) {
			html += '<td class="inactive">' + day + '</td>';
			if (++day > prevMonthDays) {
				inOldMonth = false;
				day = 1;
			}
		} else if (inNewMonth) {
			html += '<td class="inactive">' + day + '</td>';
			day++;
		} else {
			if (day == currentDay) {
				html += '<td id="selected">' + day + '</td>';
			} else {
				html += '<td>' + day + '</td>';
			}
			if (day++ == monthDays) {
				day = 1;
				inNewMonth = true;
			}
		}
		
		if (i % 7 == 6) {
			html += "</tr>";
		}
	}
	
	calendarTbody.innerHTML = html;
	
	markCurrentDay(startDate, day);
	
	updateForms();
	var endTimer = new Date();
	var millis = endTimer.getMilliseconds() - startTimer.getMilliseconds();
	log('Rendered in '+millis+'ms');
}*/

/* New version */
function drawCalendar() {
//	var startTimer = new Date();
	var html = '';
	// Figure out in which week the first of this month starts
	var weekNumber = getWeekNumber(currentYear, currentMonth, 1);
	// Figure out which number from the /previous/ month to start with
	var date = new Date(currentYear, currentMonth, 1);
	var weekday = ((date.getDay() + 6) % 7) + 1;
	var prevMonthDays = getDaysInMonth(currentMonth-1, currentYear); // don't worry about year, dec == 31 no matter what
	var monthDays = getDaysInMonth(currentMonth, currentYear);
	
	var startDate = prevMonthDays - weekday + 2;
	if (weekday == 1) { // monday, subtract 7
		startDate -= 7;
		weekNumber--;
	}
	var firstWeek = weekNumber;
	if (firstWeek == 0) {
		firstWeek = getWeekNumber(currentYear - 1, 11, 31);
	}
	var x;
	html += '<tr><th class="week">' + firstWeek + '</th>';
	weekNumber++;
	for (x = startDate; x <= prevMonthDays; x++) {
		html += '<td class="inactive">' + x + '</td>';
	}
	
	var i = prevMonthDays - startDate + 1;
	for (x = 1; x <= monthDays; x++, i++) {
		if (i % 7 == 0) {
			if (currentMonth == 11 && x >= 29) {
				weekNumber = 1;
			}
			html += '<tr><th class="week">' + weekNumber + '</th>';
			weekNumber++;
		}
		if (x == currentDay) {
			html += '<td id="selected">' + x + '</td>';
		} else {
			html += '<td>' + x + '</td>';
		}
		if (i % 7 == 6) {
			html += "</tr>";
		}
	}
	
	for (x = 1; i < 42; x++, i++) {
		if (i % 7 == 0) {
			if (currentMonth == 11) {
				weekNumber = 1;
			}
			html += '<tr><th class="week">' + weekNumber + '</th>';
			weekNumber++;
		}
		html += '<td class="inactive">' + x + '</td>';
		if (i % 7 == 6) {
			html += "</tr>";
		}
	}
	
	calendarTbody.innerHTML = html;
	
	markCurrentDay(startDate, x);
	
	updateForms();
//	var endTimer = new Date();
//	var millis = endTimer.getMilliseconds() - startTimer.getMilliseconds();
//	log('Rendered in '+millis+'ms');
}

/**
 * Marks the current day in the calendar, IF it's visible
 */
function markCurrentDay(startDate, endDate) {
	
	if (todayYear == currentYear) {
		if (todayMonth == currentMonth - 1 && todayDay >= startDate) {
			markTodayBefore();
		} else if (todayMonth == currentMonth + 1 && todayDay <= endDate) {
			markTodayAfter();
		} else if (todayMonth == currentMonth) {
			markTodayMiddle();
		}
	} else if (todayYear == currentYear - 1) {
		if (todayMonth == 11 && currentMonth == 0 && todayDay >= startDate) {
			markTodayBefore();
		}
	} else if (todayYear == currentYear + 1) {
		if (todayMonth == 0 && currentMonth == 11 && todayDay <= endDate) {
			markTodayAfter();
		}
	}
}

/**
 * Marks the current date if it's in the previous month
 */
function markTodayBefore() {
	
	var tds = document.getElementsByTagName('td');
	var i;
	for (i = 0; i < 7; i++) {
		if (parseInt(tds[i].innerHTML) == todayDay) {
			tds[i].className += " today";
			return;
		}
	}
}

/**
 * Marks the current date if it's in the next month
 */
function markTodayAfter() {
	var tds = document.getElementsByTagName('td');
	var i;
	for (i = 41; i > 27; i--) {
		if (parseInt(tds[i].innerHTML) == todayDay) {
			tds[i].className += " today";
			return;
		}
	}
}

/**
 * Marks the current date if it's in the current month
 */
function markTodayMiddle() {
	var tds = document.getElementsByTagName('td');
	var i;
	var start = todayDay > 7 ? todayDay - 7 : 0;
	for (i = start; i < start + 14; i++) {
		if (parseInt(tds[i].innerHTML) == todayDay) {
			tds[i].className = "today";
			return;
		}
	}
}

/**
 * Updates all form fields with the current date
 */
function updateForms() {
//	var x;
	
	// Date box
//	x = document.getElementById('date-input');
	var month = currentMonth < 9 ? "0" + (currentMonth+1) : (currentMonth+1);
	var day = currentDay < 10 ? "0" + currentDay : currentDay;
	dateInput.value = currentYear + '-' + month + '-' + day;
	
//	x = document.getElementById('choosemonth-span');
	choosemonth_span.innerHTML = monthNames[currentMonth];
	
//	x = document.getElementById('chooseyear-span');
	chooseyear_span.innerHTML = currentYear;
	
//	x = document.getElementById('week-input');
	weekInput.selectedIndex = -1 + getWeekNumber(currentYear, currentMonth, currentDay);
}

function gotoDate(year, month, day) {
	currentYear = year;
	currentMonth = month;
	currentDay = day;
	drawCalendar();
}

/**
 * Event handlers
 */

/**
 * Previous year ( << )
 */
function eventPrevYear(e) {
	currentYear--;
	drawCalendar();
}

/**
 * Next year ( >> )
 */
function eventNextYear(e) {
	currentYear++;
	drawCalendar();
}

/**
 * Previous month ( < )
 */
function eventPrevMonth(e) {
	if (currentMonth == 0) {
		currentMonth = 11;
		currentYear--;
	} else {
		currentMonth--
	}
	drawCalendar();
}

/**
 * Next month ( > )
 */
function eventNextMonth(e) {
	if (currentMonth == 11) {
		currentMonth = 0;
		currentYear++;
	} else {
		currentMonth++;
	}
	drawCalendar();
}

/**
 * Go to today ( * )
 */
function eventGotoToday(e) {
	currentYear = todayYear;
	currentMonth = todayMonth;
	currentDay = todayDay;
	drawCalendar();
}

/**
 * Change week number
 */
function eventGotoWeek(e) {
	var week = e.target.selectedIndex + 1;;
	var date = new Date(currentYear, 0, 1);
	
	var totalDays = week * 7 - ((date.getDay() + 2) % 7) - 3;
	
	var month = 0;
	while (totalDays > 0) {
		totalDays -= getDaysInMonth(month++, currentYear);
	}
	date.setMonth(month);
	date.setDate(totalDays);
	
	currentYear = date.getFullYear();
	currentMonth = date.getMonth();
	currentDay = date.getDate();
	
	drawCalendar();
	e.target.blur();
}

/*
*/

function dateInputKey(event) {
	if (event.keyIdentifier == "Enter") {
		// Parse date & set it
		var date = event.target.value;
		date = date.split('-');
		if (date.length > 3 || date.length < 2) {
			return;
		}
		var year = parseInt(date[0]);
		// Fix months starting with 0
		if (date[1].substr(0,1) == '0') {
			date[1] = date[1].substr(1);
		}
		var month = parseInt(date[1]);
		if (!year || !month) {
			return;
		}
		
		var day = currentDay;
		if (date.length == 3) { // Regular date, parse day as well
			var day = parseInt(date[2]);
			if (!day) {
				return;
			}
		}
		var max = getDaysInMonth(month-1, year);
		if (day > max) {
			day = max;
		}
		currentDay = day;
		currentMonth = month - 1;
		currentYear = year;
		drawCalendar();
		event.stopPropagation();
		event.target.blur();
	}
}

function yearInputKey(event) {
	if (event.keyIdentifier == "Enter") {
		var el = event.target;
		el.onblur = null;
		el.blur();
		hideYearInput(event);
	}
}

function hideYearInput(event) {
	event.stopPropagation();
	event.preventDefault();
//	var el = document.getElementById('chooseyear-span');
	chooseyear_span.style.display = "block";
//	el = document.getElementById('chooseyear-input');
	chooseyear_input.style.display = "none";
	// Update year
	var year = parseInt(chooseyear_input.value);
	if (year) {
		currentYear = year;
		drawCalendar();
	}
}

function showYearInput(event) {
//	var el = document.getElementById('chooseyear-span');
	chooseyear_span.style.display = "none";
//	el = document.getElementById('chooseyear-input');
	chooseyear_input.value = currentYear;
	chooseyear_input.onblur = hideYearInput;
	chooseyear_input.style.display = "block";
	chooseyear_input.focus();
	chooseyear_input.select();
}

function hideMonthInput(event) {
	event.stopPropagation();
	event.preventDefault();
	var span = document.getElementById('choosemonth-span');
	var input = document.getElementById('choosemonth-input');
	input.blur();
	input.style.display = "none";
	var index = input.selectedIndex;
	var month = monthNames[index];
	span.innerHTML = month;
	span.style.display = "block";
	currentMonth = index;
	drawCalendar();
}

function showMonthInput(event) {
	var el = document.getElementById('choosemonth-span');
	el.style.display = "none";
	el = document.getElementById('choosemonth-input');
	el.selectedIndex = currentMonth;
	el.onchange = hideMonthInput;
	el.onblur = hideMonthInput;
	el.style.display = "block";
	el.focus();
}

function selectDate() {
	var tds = document.getElementsByTagName('td');
	var start = (currentDay < 7) ? 0 : currentDay - 7;
	var toSelect = null;
	for (i = start; i < start + 14; i++) {
		if (parseInt(tds[i].innerHTML) == currentDay) {
			toSelect = tds[i];
			break;
		}
	}
	if (toSelect != null) {
		var orig = document.getElementById('selected');
		orig.id = '';
		toSelect.id = 'selected';
	}
	updateForms();
}

function keyHandler(e) {
	
	if (e.target.tagName == "INPUT") {
		return;
	}
	
	var changed = false;
	switch (e.keyIdentifier) {
		case 'Up':
			if (currentDay > 7) {
				currentDay -= 7;
				selectDate();
			} else {
				// Descend into previous month
				if (currentMonth == 0) {
					currentYear--;
					currentMonth = 11;
				} else {
					currentMonth--;
				}
				currentDay = getDaysInMonth(currentMonth, currentYear) - 7 + currentDay;
				drawCalendar();
			}
			changed = true;
			break;
		
		case 'Down':
			var days = getDaysInMonth(currentMonth, currentYear);
			if (currentDay <= days - 7) {
				currentDay += 7;
				selectDate();
			} else {
				// Ascend into next month
				if (currentMonth == 11) {
					currentYear++;
					currentMonth = 0;
				} else {
					currentMonth++;
				}
				currentDay = currentDay - days + 7;
				drawCalendar();
			}
			changed = true;
			break;
		
		case 'Left':
			if (currentDay == 1) {
				if (currentMonth == 0) {
					currentYear--;
					currentMonth = 11;
				} else {
					currentMonth--;
				}
				currentDay = getDaysInMonth(currentMonth, currentYear);
				drawCalendar();
			} else {
				currentDay--;
				selectDate();
			}
			changed = true;
			break;
		
		case 'Right':
			if (currentDay == getDaysInMonth(currentMonth, currentYear)) {
				if (currentMonth == 11) {
					currentYear++;
					currentMonth = 0;
				} else {
					currentMonth++;
				}
				currentDay = 1;
				drawCalendar();
			} else {
				currentDay++;
				selectDate();
			}
			changed = true;
			break;
	}
	
	if (changed) {
		e.stopPropagation();
		e.preventDefault();
	}
}

function setSelected(e) {
	var td = e.target;
	if (td.tagName != 'TD') {
		return;
	}
	var orig = document.getElementById('selected');
	orig.id = '';
	
	currentDay = parseInt(td.innerHTML);
	if (td.className.substr(0, 8) == "inactive") {
		
		if (currentDay < 16) {
			// Next month
			if (currentMonth == 11) {
				currentMonth = 0;
				currentYear++;
			} else {
				currentMonth++;
			}
		} else {
			if (currentMonth == 0) {
				currentMonth = 11;
				currentYear--;
			} else {
				currentMonth--;
			}
		}
		drawCalendar();
	} else {
		td.id = 'selected';
		
		updateForms();
	}
}

function setToday(isInit) {
	var div = document.getElementById('today');
	var today = new Date();
	var day = dayNames[today.getDay()];
	var number = today.getDate();
	var month = monthNames[today.getMonth()];
	div.innerHTML = '<span class="today-day">' + day + '</span><span class="today-date">'
		+ number + '</span><span class="today-month">' + month + '</span>';
	
	if (isInit) {
		// Set global variables
		todayYear  = currentYear  = today.getFullYear();
		todayMonth = currentMonth = today.getMonth();
		todayDay   = currentDay   = today.getDate();
		
		// Draw calender properly
		drawCalendar();
	}
}

function setEventHandlers() {
	var x;
	
	// Top row: prev/next year/month
	x = document.getElementById('prevyear');
	x.addEventListener('click', eventPrevYear, false);
	
	x = document.getElementById('nextyear');
	x.addEventListener('click', eventNextYear, false);
	
	x = document.getElementById('prevmonth');
	x.addEventListener('click', eventPrevMonth, false);
	
	x = document.getElementById('nextmonth');
	x.addEventListener('click', eventNextMonth, false);
	
	x = document.getElementById('chooseyear-span');
	x.addEventListener('click', showYearInput, false);
	
	x = document.getElementById('chooseyear-input');
	x.addEventListener('keypress', yearInputKey, false);
	
	x = document.getElementById('choosemonth-span');
	x.addEventListener('click', showMonthInput, false);
	
	// Bottom row
	x = document.getElementById('goto-today');
	x.addEventListener('click', eventGotoToday, false);
	
	x = document.getElementById('week-input');
	x.addEventListener('change', eventGotoWeek, false);
	
	x = document.getElementById('date-input');
	x.addEventListener('keypress', dateInputKey, false);
	
	// Calendar field
	x = document.getElementById('tbody');
	x.addEventListener('click', setSelected, false);
	
	window.addEventListener('keypress', keyHandler, false);
}

function onshow() {
	// Check if the "today" date is still valid
	var date = new Date();
	if (todayDay != date.getDate() || todayMonth != date.getMonth()
		|| todayYear != date.getFullYear()) {
		setToday(false);
		// Update date
		if (currentDay == todayDay && currentMonth == todayMonth && currentYear == todayYear) {
			// Update current date as well
			todayYear  = currentYear  = date.getFullYear();
			todayMonth = currentMonth = date.getMonth();
			todayDay   = currentDay   = date.getDate();
		} else {
			todayYear  = date.getFullYear();
			todayMonth = date.getMonth();
			todayDay   = date.getDate();
		}
		drawCalendar();
	}
}


function defocus(event) {
	event.target.blur();
	event.target.removeEventListener('focus', defocus, false);
}

function fillUpGlobals() {
	calendarTbody = document.getElementById('tbody');
	dateInput = document.getElementById('date-input');
	weekInput = document.getElementById('week-input');
	
	choosemonth_input = document.getElementById('choosemonth-input');
	choosemonth_span = document.getElementById('choosemonth-span');
	
	chooseyear_input = document.getElementById('chooseyear-input');
	chooseyear_span = document.getElementById('chooseyear-span');
	
}

function setup() {
	fillUpGlobals();
	
	setToday(true);
	
	setEventHandlers();
	
	if (window.widget) {
		var x = document.getElementById('date-input');
		x.addEventListener('focus', defocus, false);
		widget.onshow = onshow;
	}
}

function log(msg) {
	var x = document.getElementById('status');
	x.innerHTML = '<div>'+msg+'</div>' + x.innerHTML;
}

window.onload = setup;
